#!/usr/bin/perl

# === Usage - tile rendering ===
# $ ./tile_rendering.pl -r image_width image_height num_tiles_x num_tiles_y margin_x margin_y program_name num_machines machine_idx renderer_arguments
#
# Assume you want to distribute the brute force renderer to 5 machines, the
# image dimension is 512x512, there are 5x4 tiles, and the margin is 40 in x
# and 20 in y, launch
# this command on the first machine:
# $ ./tile_rendering.pl -r 512 512 5 4 40 20 brute_force 5 0 " --gaussian -c car.config -t 1 -s 1"
#
# Similarily, run this command on the second machine
# $ ./tile_rendering.pl -r 512 512 5 4 40 20 brute_force 5 1 " --gaussian -c car.config -t 1 -s 1"
#
# === Usage - tile merging ===
# $ ./tile_rendering.pl -m program_name image_width image_height num_tiles_x num_tiles_y margin_x margin_y
# For example
# $ ./tile_rendering.pl -m program_name image_width image_height num_tiles_x num_tiles_y margin_x margin_y
# The output is at merged.exr

my $dirname = `dirname $0`;
chomp $dirname;

my $num_machines;
my $machine_idx;
my $is_running_renderer = 0;
my $is_merging = 0;
my $program_name;
my $renderer_args;

my $image_width = $ARGV[1];
my $image_height = $ARGV[2];
my $num_tiles_x = $ARGV[3];
my $num_tiles_y = $ARGV[4];
my $margin_x = $ARGV[5];
my $margin_y = $ARGV[6];

print "Settings:\n";
print "Image dimension: ${image_width}x$image_height\n";
print "Num tiles in x and y: $num_tiles_x, $num_tiles_y\n";
print "Sampling margin in x and y: $margin_x, $margin_y\n";

if ($ARGV[0] eq "-r") {
  $is_running_renderer = 1;
  $program_name = $ARGV[7];
  $num_machines = $ARGV[8];
  $machine_idx = $ARGV[9];
  $renderer_args = $ARGV[10];
  print "Running renderer. Distribute the jobs to $num_machines machines. ".
        "Now running the $machine_idx-th task.\n";
}
if ($ARGV[0] eq "-m") {
  $is_merging = 1;
  print "Merging images\n";
  die "Can't find merge_exr binary." unless -e "$dirname/merge_exr";
}

open MERGE_PROGRAM, "| $dirname/merge_exr" if $is_merging;
print MERGE_PROGRAM "$image_width $image_height\n" if $is_merging;

for (my $xi = 0; $xi < $num_tiles_x; $xi++) {
  for (my $yi = 0; $yi < $num_tiles_y; $yi++) {
    my $left = int($xi / $num_tiles_x * $image_width);
    my $right = int(($xi+1) / $num_tiles_x * $image_width);
    my $top = int($yi / $num_tiles_y * $image_height);
    my $bottom = int(($yi+1) / $num_tiles_y * $image_height);
    $marge_left = $left - $margin_x;
    $marge_right = $right + $margin_x;
    $marge_top = $top - $margin_y;
    $marge_bottom = $bottom + $margin_y;
    $marge_left = 0 if $marge_left < 0;
    $marge_top = 0 if $marge_top < 0;
    $marge_right = $image_width - 1 if $marge_right >= $image_width;
    $marge_bottom = $image_height - 1 if $marge_bottom >= $image_height;

    my $filename = "output/tile-$xi-$yi-$num_tiles_x-$num_tiles_y.exr";
    if ($is_running_renderer) {
      next unless ($xi + $yi * $num_tiles_x) % $num_machines == $machine_idx;
      my $cmd = "$dirname/$program_name --mrr $marge_top,$marge_bottom,$marge_left,$marge_right -o $filename $renderer_args";
      print "$cmd\n";
      system ($cmd);
    }

    if ($is_merging) {
      print MERGE_PROGRAM "$filename $top $bottom $left $right\n" if $is_merging and -e $filename;
      print "$filename not found.\n" unless $is_merging and -e $filename;
    }
  }
}

close MERGE_PROGRAM if $is_merging;
